/*
 * Code to control a LED-board from a VNC client
 * Copyright (c) 2013 Sprite_tm <jeroen@spritesmods.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "frame.h"
#include "dmaloop.h"

//Connections for the LED-screen
#define LED_R0 (1<<8)
#define LED_G0 (1<<7)
#define LED_R2 (1<<25)
#define LED_G2 (1<<11)
#define LED_R1 (1<<10)
#define LED_G1 (1<<9)
#define LED_STROBE (1<<17)
#define LED_A3 (1<<23)
#define LED_A2 (1<<22)
#define LED_A1 (1<<27)
#define LED_A0 (1<<18)
#define LED_RENA (1<<24)
#define LED_GENA (1<<24)
#define LED_CLK (1<<4)

#define LED_IO (LED_G0|LED_R0|LED_G1|LED_R1|LED_G2|LED_R2|LED_STROBE|LED_A0|LED_A1|LED_A2|LED_A3|LED_RENA|LED_GENA|LED_CLK)

//Width of a frame
#define FRAMEW 224

//Macros to get the value of a pixel in the pixelmap.
#define PIXELR(map, x, y) map[(((x)+((y)*FRAMEW))*3)]
#define PIXELG(map, x, y) map[(((x)+((y)*FRAMEW))*3)+1]
#define PIXELB(map, x, y) map[(((x)+((y)*FRAMEW))*3)+2]

//Grayscale generation parameters
#define GRAYINC 16
#define GRAYSPREADX 128
#define GRAYSPREADY 64

void frameInit() {
	dmaloopInit(LED_IO);
}

//Deeper frame-generation magic here.
void frameSend(unsigned char *pixmap) {
	int x,y,b,lb,oldy=15;
	unsigned char fcnt=0; //per-frame grayscale threshold
	unsigned char lcnt, pcnt, ppcnt;
	int pos=0;

	do {
		fcnt+=GRAYINC;
		lcnt=fcnt; //set per-line grayscale threshold
	
		for (y=0; y<16; y++) {
			lcnt+=GRAYINC+GRAYSPREADX;
			lb=0;
			if (oldy&1) lb|=LED_A0;
			if (oldy&2) lb|=LED_A1;
			if (oldy&4) lb|=LED_A2;
			if (oldy&8) lb|=LED_A3;
			ppcnt=lcnt;
			for (x=0; x<FRAMEW; x++) {
				ppcnt+=GRAYSPREADY+GRAYINC; 
				pcnt=ppcnt;
				if ((x+y)&1) pcnt+=8; //Location-dependent dithering: every even pixel is 0.5 grayscales lower
				b=lb;
				if (x==FRAMEW-1) b|=LED_STROBE;
//				pcnt=127;
				if (PIXELR(pixmap, x, y)>pcnt) b|=LED_R0;
				if (PIXELG(pixmap, x, y)>pcnt) b|=LED_G0;
				if (PIXELR(pixmap, x, y+16)>pcnt) b|=LED_R1;
				if (PIXELG(pixmap, x, y+16)>pcnt) b|=LED_G1;
				if (PIXELR(pixmap, x, y+32)>pcnt) b|=LED_R2;
				if (PIXELG(pixmap, x, y+32)>pcnt) b|=LED_G2;
				if (y==15 && x==FRAMEW-1) {
					b|=LED_RENA+LED_GENA;
				}
				dmaloopWriteWord(pos++, b);
				dmaloopWriteWord(pos++, b|LED_CLK);
			}
			oldy=y;
		}
	} while (fcnt!=0);
}
