/*
 * Code to control a LED-board from a VNC client
 * Copyright (c) 2013 Sprite_tm <jeroen@spritesmods.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "frame.h"
#include "pwmcorr.h"
#include <gd.h>
#include <sys/time.h>
#include <time.h>
#include <rfb/rfbclient.h>

unsigned char framebuffer[224*48*3];
int offx=50, offy=50; //Offset in the server framebuffer we display

//libvncclient calls this after it has received an update for part of the screen.
//We use it to update the LED framebuffer.
static void updateFramebuffer(rfbClient* client, int ux, int uy, int uw, int uh) {
	int x, y, i;
	char *p, *q;
	int changed=0;
	for (y=uy; y<uy+uh; y++) {
		p=client->frameBuffer+((client->width*y)+ux)*4;
		q=framebuffer+((224*(y-offy))+(ux-offx))*3;
		for (x=ux; x<ux+uw; x++) {
			//Update a line. Only get the pixels inside our own framebuffer.
			if (x-offx<224 && y-offy<48 && x-offx>=0 && y-offy>=0) {
				//Copy the R, G, B values.
				for (i=0; i<3; i++) {
					*q=pwmcorr(*p); //Do PWM value correction here.
					p++; q++;
				}
				p++;
				changed=1;
			} else {
				//Skip pixel.
				p+=4; q+=3;
			}
		}
	}
	if (changed) frameSend(framebuffer);
}

//Send the contents of a gdImagePtr to the leds.
static void fbSendGdImage(gdImagePtr im) {
	int x,y,p;
	for (y=0; y<48; y++) {
		for (x=0; x<224; x++) {
			p=gdImageGetPixel(im, x, y);
			framebuffer[(x+(y*224))*3]=pwmcorr(gdImageRed(im, p));
			framebuffer[(x+(y*224))*3+1]=pwmcorr(gdImageGreen(im, p));
			framebuffer[(x+(y*224))*3+2]=pwmcorr(gdImageBlue(im, p));
		}
	}
	frameSend(framebuffer);
}


//Display the given text on the LEDs
static void fbPutText(char *str) {
	gdImagePtr im;
	int bg, fg;
	im=gdImageCreateTrueColor(224,48);
	bg=gdImageColorAllocate(im, 0,0,0);
	fg=gdImageColorAllocate(im, 255,0,0);
	gdImageString(im, gdFontGetLarge(), 0, 0, str, fg);
	fbSendGdImage(im);
	gdImageDestroy(im);
}

int main(int argc, char** argv) {
	//Arguments for libvncserver.
	const char *libArgv[]={"led", "-listennofork"};
	int libArgc=2;
	rfbClient* client;

	frameInit();
	client=rfbGetClient(8,3,4); //32-bit client (24-bit borks somehow...)

	while(1) {
		//Display welcome banner
		if (argc>1) fbPutText(argv[1]);
		//Initialize a connection.
		client->GotFrameBufferUpdate=updateFramebuffer;
		client->listenPort=5500;
		if (!rfbInitClient(client,&libArgc, libArgv)) return 1;

		//Go handle VNC stuff until the connection is broken.
		while(1) {
			if(WaitForMessage(client,50)<0) break;
			if(!HandleRFBServerMessage(client)) break;
		}
	}
	return 0;
}
